package com.pj.api;


import cn.dev33.satoken.annotation.SaCheckPermission;
import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.log.StaticLog;
import com.pj.current.satoken.StpUserUtil;
import com.pj.project.ocr.OcrService;
import com.pj.project.ocr.bo.OcrResult;
import com.pj.project.re_mini_user_car_filing.ReMiniUserCarFiling;
import com.pj.project.re_mini_user_car_filing.ReMiniUserCarFilingService;
import com.pj.project.re_mini_user_person_filing.ReMiniUserPersonFiling;
import com.pj.project.re_mini_user_person_filing.ReMiniUserPersonFilingService;
import com.pj.project.sp_admin.SpAdminService;
import com.pj.project.tb_car_filing.TbCarFiling;
import com.pj.project.tb_car_filing.TbCarFilingService;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_mini_user.TbMiniUserService;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import com.pj.project.tb_person_visit_record.TbPersonVisitRecordService;
import com.pj.project.wx.WxService;
import com.pj.utils.sg.AjaxError;
import com.pj.utils.sg.AjaxJson;
import com.pj.utils.so.SoMap;
import lombok.extern.slf4j.Slf4j;
import org.aspectj.weaver.loadtime.Aj;
import org.springframework.web.bind.annotation.*;

import javax.annotation.Resource;
import java.util.HashMap;
import java.util.List;
import java.util.Map;
import java.util.stream.Collectors;

@RequestMapping("wx")
@RestController
@Slf4j
public class WxController {
    @Resource
    private WxService wxService;
    @Resource
    private TbMiniUserService tbMiniUserService;
    @Resource
    private TbPersonFilingService tbPersonFilingService;
    @Resource
    private ReMiniUserPersonFilingService reMiniUserPersonFilingService;
    @Resource
    private TbCarFilingService tbCarFilingService;
    @Resource
    private ReMiniUserCarFilingService reMiniUserCarFilingService;
    @Resource
    private TbPersonVisitRecordService tbPersonVisitRecordService;

    @Resource
    private SpAdminService spAdminService;

    @Resource
    private OcrService ocrService;

    @Resource
    private TbDeptService tbDeptService;


    @RequestMapping(value = "getNeedJudgeNum")
    public AjaxJson getNeedJudgeNum() {
        Map<String,Integer>result=new HashMap<>();
        int num=0;
        int personNum=0;
        if (StpUserUtil.isAdmin()){
            num = tbCarFilingService.getNeedJudgeNum(null);
            personNum = tbPersonFilingService.getNeedJudgeNum(null);
        }else  if (StpUserUtil.isJudgeAdmin()) {
             num = tbCarFilingService.getNeedJudgeNum(StpUserUtil.getDepartmentId());
             personNum = tbPersonFilingService.getNeedJudgeNum(StpUserUtil.getDepartmentId());
        }
        result.put("carNum",num);
        result.put("personNum",personNum);
        return AjaxJson.getSuccessData(result);
    }

    @GetMapping(value = "getRedirectUrl")
    public AjaxJson getRedirectUrl(String path, @RequestParam(required = false) String state) {
        return AjaxJson.getSuccessData(wxService.getRedirectUrl(path, state));
    }

    @GetMapping(value = "getOpenidByCode")
    public AjaxJson getOpenidByCode(String code, @RequestParam(required = false) String openid) {
        return AjaxJson.getSuccessData(wxService.getOpenidByCode(code, openid));
    }


    @GetMapping(value = "getWxConfig")
    public AjaxJson getWxConfig(String url) {
        return AjaxJson.getSuccessData(wxService.getWxConfig(url));
    }

    /**
     * 获取人员所属组织
     *
     * @return
     */
    @GetMapping("getOwnerDeptList")
    public AjaxJson getDeptList() {
        String departmentId = StpUserUtil.getDepartmentId();
        List<TbDept> list = tbDeptService.list();
        if (StpUserUtil.isAdmin()) {
            return AjaxJson.getSuccessData(list);
        } else if (StpUserUtil.isJudgeAdmin()) {
            list = list.stream().filter(tbDept -> StrUtil.equals(tbDept.getDepartmentId(), departmentId)).collect(Collectors.toList());
        } else {
            List<Long> deptIdList = StpUserUtil.getDeptIdList();
            list = list.stream().filter(tbDept -> deptIdList.contains(tbDept.getId())).collect(Collectors.toList());
        }
        return AjaxJson.getSuccessData(list);
    }

    /**
     * 获取所有的组织
     *
     * @return
     */
    @GetMapping("getAllDeptList")
    public AjaxJson getAllDeptList() {
        List<TbDept> list = tbDeptService.list();
        return AjaxJson.getSuccessData(list);
    }


    @GetMapping(value = "person-filing")
    public AjaxJson getPersonFiling() {
        SoMap so = SoMap.getRequestSoMap();
        if (StpUserUtil.isAdmin()) {//超级管理员---看所有
            List<TbPersonFiling> list = tbPersonFilingService.getList(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        } else if (StpUserUtil.isJudgeAdmin()) {//审核人员---看改单位下的所有
            so.put("departmentId", StpUserUtil.getDepartmentId());
            List<TbPersonFiling> list = tbPersonFilingService.getList(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        } else {
            so.put("miniUserId", StpUserUtil.getMiniUserId());
            //看到自己备案的
            List<TbPersonFiling> list = tbPersonFilingService.getOuterList(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        }
    }

    /**
     * 添加备案
     *
     * @param tbPersonFiling
     * @return
     */
    @PostMapping("addPersonFiling")
    public AjaxJson addPersonFiling(TbPersonFiling tbPersonFiling) {
        if (!StpUtil.isLogin()) {
            throw new AjaxError("请先登录");
        }
        if (StrUtil.isEmpty(tbPersonFiling.getPhoto())){
            return AjaxJson.getError("请上传人脸");
        }
        TbPersonFiling db = tbPersonFilingService.findByIdCard(tbPersonFiling.getIdCard());
        if (db != null) {
            tbPersonFiling = db;
        } else {
            tbPersonFilingService.add(tbPersonFiling);
        }
        ReMiniUserPersonFiling reMiniUserPersonFiling = new ReMiniUserPersonFiling();
        reMiniUserPersonFiling.setFilingPersonId(tbPersonFiling.getId())
                .setMiniUserId(StpUserUtil.getMiniUserId());
        reMiniUserPersonFilingService.save(reMiniUserPersonFiling);
        return AjaxJson.getSuccess();
    }

    @RequestMapping("judgePersonFiling")
    public AjaxJson judge(TbPersonFiling filing) {
        tbPersonFilingService.judge(filing);
        return AjaxJson.getSuccess();
    }

    @RequestMapping("judgeCarFiling")
    public AjaxJson judgeCarFiling(TbCarFiling filing) {
        tbCarFilingService.judge(filing);
        return AjaxJson.getSuccess();
    }

    /**
     * 编辑备案
     *
     * @param tbPersonFiling
     * @return
     */
    @PostMapping("editPersonFiling")
    public AjaxJson editPersonFiling(TbPersonFiling tbPersonFiling) {
        if (StrUtil.isEmpty(tbPersonFiling.getPhoto())){
            return AjaxJson.getError("请上传人脸");
        }
        tbPersonFilingService.update(tbPersonFiling);
        return AjaxJson.getSuccess();
    }

    /**
     * 删除备案
     *
     * @param id
     * @return
     */
    @PostMapping("deletePersonFiling")
    public AjaxJson deletePersonFiling(Integer id) {
        tbPersonFilingService.delete(id);
        return AjaxJson.getSuccess();
    }

    /**
     * 添加车辆备案
     *
     * @param tbCarFiling
     * @return
     */
    @PostMapping("addCarFiling")
    public AjaxJson addCarFiling(TbCarFiling tbCarFiling) {
        tbCarFilingService.add(tbCarFiling);
        ReMiniUserCarFiling reMiniUserCarFiling = new ReMiniUserCarFiling();
        reMiniUserCarFiling.setFilingCarId(tbCarFiling.getId())
                .setMiniUserId(StpUserUtil.getMiniUserId());
        reMiniUserCarFilingService.save(reMiniUserCarFiling);
        return AjaxJson.getSuccess();
    }

    /**
     * 编辑车辆备案
     *
     * @param tbCarFiling
     * @return
     */
    @PostMapping("editCarFiling")
    public AjaxJson editCarFiling(TbCarFiling tbCarFiling) {
        tbCarFilingService.update(tbCarFiling);
        return AjaxJson.getSuccess();
    }

    /**
     * 获取车辆备案列表
     *
     * @return
     */
    @GetMapping("car-filing")
    public AjaxJson getCarFiling() {
        SoMap so = SoMap.getRequestSoMap();
        StaticLog.info("is admin:{},{}", StpUserUtil.isAdmin(), StpUserUtil.isJudgeAdmin());
        if (StpUserUtil.isAdmin()) {//超级管理员---看所有
            List<TbCarFiling> list = tbCarFilingService.getList(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        } else if (StpUserUtil.isJudgeAdmin()) {//审核人员---看改单位下的所有
            so.put("departmentId", StpUserUtil.getDepartmentId());
            List<TbCarFiling> list = tbCarFilingService.getList(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        } else {
            so.put("miniUserId", StpUserUtil.getMiniUserId());
            //看到自己备案的
            List<TbCarFiling> list = tbCarFilingService.getOuterList(so.startPage());
            return AjaxJson.getPageData(so.getDataCount(), list);
        }
    }

    @PostMapping("deleteCarFiling")
    public AjaxJson deleteCarFiling(Long id) {
        tbCarFilingService.delete(id);
        return AjaxJson.getSuccess();
    }

    @PostMapping("loginByOpenid")
    public AjaxJson loginByOpenid() {
        SoMap soMap = SoMap.getRequestSoMap();
        String openid = soMap.getString("openid");
        Integer type = soMap.getInt("type");
        if (StrUtil.isEmpty(openid)) {
            return AjaxJson.getError("参数异常");
        }
        return tbMiniUserService.loginByOpenid(openid, type);
    }

    @RequestMapping("solve")
    public AjaxJson solve() {
        SoMap soMap = SoMap.getRequestSoMap();
        tbPersonVisitRecordService.solve(soMap);
        return AjaxJson.getSuccess();
    }

    /**
     * 绑定
     *
     * @param openid
     * @param key
     * @param password
     * @param type
     * @param name
     * @return
     */
    @PostMapping("bind")
    public AjaxJson bind(String openid, String key, String password, Integer type, String name, Long deptId) {
        return spAdminService.bindUser(openid, key, password, type, name, deptId);
    }

    /**
     * 登录
     *
     * @param openid
     * @param key
     * @param password
     * @return
     */
    @PostMapping("login")
    public AjaxJson login(String openid, String key, String password) {
        return tbMiniUserService.login(openid, key, password);
    }

    /**
     * 识别边民证，弃用
     */
    @RequestMapping("ocrIcCard")
    public AjaxJson ocrIcCard(String icCardUrl) {
        OcrResult ocrResult = ocrService.ocrIcCard(icCardUrl);
        return AjaxJson.getSuccessData(ocrResult);
    }

    /**
     * 识别身份证，弃用
     */
    @RequestMapping("ocrIdCard")
    @Deprecated
    public AjaxJson ocrIdCard(String idCardUrl) {
        OcrResult ocrResult = ocrService.ocrIdCard(idCardUrl);
        return AjaxJson.getSuccessData(ocrResult);
    }

    /**
     * 识别身份证或边民证
     */
    @RequestMapping("ocrIdOrIcCard")
    public AjaxJson ocrIdOrIcCard(String cardUrl) {
        OcrResult ocrResult = ocrService.ocrIdOrIc(cardUrl);
        return AjaxJson.getSuccessData(ocrResult);
    }

}
