package com.pj.biz;

import java.util.List;

import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_mini_user.TbMiniUser;
import com.pj.project.tb_mini_user.TbMiniUserService;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.transaction.annotation.Transactional;
import org.springframework.web.bind.annotation.*;

import com.pj.utils.sg.*;
import com.pj.utils.so.*;
import com.pj.project4sp.SP;

import com.pj.current.satoken.StpUserUtil;
import cn.dev33.satoken.annotation.SaCheckPermission;


/**
 * Controller: tb_mini_user -- h5端用户
 *
 * @author qzyReal
 */
@RestController
@RequestMapping("/TbMiniUser/")
public class TbMiniUserController {

    /**
     * 底层 Service 对象
     */
    @Autowired
    TbMiniUserService tbMiniUserService;

    /**
     * 增
     */
    @RequestMapping("add")
    @SaCheckPermission(TbMiniUser.PERMISSION_CODE)
    @Transactional(rollbackFor = Exception.class)
    public AjaxJson add(TbMiniUser t) {
        tbMiniUserService.add(t);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 删
     */
    @RequestMapping("delete")
    @SaCheckPermission(TbMiniUser.PERMISSION_CODE)
    public AjaxJson delete(Long id) {
        tbMiniUserService.delete(id);
        return AjaxJson.getSuccess();
    }

    /**
     * 删 - 根据id列表
     */
    @RequestMapping("deleteByIds")
    @SaCheckPermission(TbMiniUser.PERMISSION_CODE)
    public AjaxJson deleteByIds() {
        List<Long> ids = SoMap.getRequestSoMap().getListByComma("ids", long.class);
        int line = SP.publicMapper.deleteByIds(TbMiniUser.TABLE_NAME, ids);
        return AjaxJson.getByLine(line);
    }

    /**
     * 改
     */
    @RequestMapping("update")
    public AjaxJson update(TbMiniUser t) {
        tbMiniUserService.update(t);
        return AjaxJson.getSuccess();
    }

    /**
     * 查 - 根据id
     */
    @RequestMapping("getById")
    public AjaxJson getById(Long id) {
        TbMiniUser t = tbMiniUserService.getById(id);
        return AjaxJson.getSuccessData(t);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    @RequestMapping("getList")
    public AjaxJson getList() {
        SoMap so = SoMap.getRequestSoMap();
        if (!StpUserUtil.isAdmin()) {
            so.put("deptIdList",StpUserUtil.getDeptIdList());
        }
        List<TbMiniUser> list = tbMiniUserService.getList(so.startPage());
        return AjaxJson.getPageData(so.getDataCount(), list);
    }
    @RequestMapping("updateDept")
    @SaCheckPermission(TbMiniUser.PERMISSION_CODE_EDIT)
    public AjaxJson updateDept() {
        SoMap so = SoMap.getRequestSoMap();
       Long id= so.getLong("id");
       Long deptId= so.getLong("deptId");
       tbMiniUserService.updateDept(id,deptId);

        return AjaxJson.getSuccess();
    }
    @RequestMapping("updatePassword")
    @SaCheckPermission(TbMiniUser.PERMISSION_CODE_EDIT)
    public AjaxJson updatePassword() {
        SoMap so = SoMap.getRequestSoMap();
       Long id= so.getLong("id");
       String password= so.getString("password");
       tbMiniUserService.updatePassword(id,password);
        return AjaxJson.getSuccess();
    }


}
