package com.pj.project4sp.admin4acc;

import javax.servlet.http.HttpServletRequest;

import cn.hutool.cache.CacheUtil;
import cn.hutool.cache.impl.TimedCache;
import cn.hutool.core.util.StrUtil;
import cn.hutool.crypto.asymmetric.KeyType;
import cn.hutool.crypto.asymmetric.RSA;
import com.pj.utils.cache.RedisUtil;
import com.wf.captcha.ArithmeticCaptcha;
import com.wf.captcha.SpecCaptcha;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import com.alibaba.csp.sentinel.annotation.SentinelResource;
import com.alibaba.csp.sentinel.slots.block.BlockException;
import com.pj.project.sp_admin.SpAdmin;
import com.pj.project.sp_admin.SpAdminUtil;
import com.pj.project4sp.role4permission.SpRolePermissionService;
import com.pj.project4sp.spcfg.SpCfgUtil;
import com.pj.utils.sg.AjaxJson;
import com.pj.utils.sg.NbUtil;
import com.pj.utils.so.SoMap;

import cn.dev33.satoken.stp.StpUtil;

import java.util.HashMap;
import java.util.Map;
import java.util.UUID;

/**
 * Admin账号相关的接口 
 * 
 * @author kong
 *
 */
@RestController
@RequestMapping("/AccAdmin/")
public class SpAccAdminController {

	@Autowired
    SpAccAdminService spAccAdminService;
	
	@Autowired
	SpRolePermissionService spRolePermissionService;
	
	/** 账号、密码登录  */
	@RequestMapping("doLogin")
	@SentinelResource(value = "qps-max-1", blockHandler = "doLoginBlock")
	AjaxJson doLogin(String key, String password, String code, String verCode) {
		// 1、验证参数
		if (NbUtil.hasNull(key, password)) {
			return AjaxJson.getError("请提供key与password参数");
		}

		String cacheCode = RedisUtil.get(code);
		if (NbUtil.isNull(cacheCode)) {
			return AjaxJson.getError("验证码已过期");
		}
		RSA rsa = KEY_CACHE.get(code);
		if (rsa==null){
			return AjaxJson.getError("请重新刷新登录");
		}
		if (!StrUtil.equals(cacheCode.toUpperCase(), verCode.toUpperCase())) {
			return AjaxJson.getError("验证码不正确");
		}

		password=rsa.decryptStr(password, KeyType.PrivateKey);
		RedisUtil.del(code);
		return spAccAdminService.doLogin(key, password);
	}
	private TimedCache<String, RSA> KEY_CACHE = CacheUtil.newTimedCache(60000);
	@PostMapping("captcha")
	public AjaxJson render() {
		SpecCaptcha captcha = new SpecCaptcha(130, 48,4);
		String key = UUID.randomUUID().toString();
		String verCode = captcha.text().toLowerCase();   // 获取验证码的字符
		RedisUtil.setBySECONDS(key, verCode, 60);
		Map<String, String> map = new HashMap<>();
		map.put("key", key);
		map.put("image", captcha.toBase64());
		RSA rsa = new RSA();
		String publicKeyBase64Key = rsa.getPublicKeyBase64();
		KEY_CACHE.put(key, rsa);
		map.put("pKey", publicKeyBase64Key);
		return AjaxJson.getSuccessData(map);
	}

	// 限流之后触发的函数 
	AjaxJson doLoginBlock(String key, String password, BlockException e) {
		return AjaxJson.getError("访问过于频繁，请稍后再试");
	}
	
	/** 退出登录  */
	@RequestMapping("doExit")
	AjaxJson doExit() {
		StpUtil.logout();
		return AjaxJson.getSuccess();
	}
	
	/** 获取会话信息 */
	@RequestMapping("getLoginInfo")
	AjaxJson getLoginInfo(HttpServletRequest request) {
		// 当前admin
		SpAdmin admin = SpAdminUtil.getCurrAdmin();
		
		// 组织参数 (admin信息，权限信息，配置信息)
		SoMap map = new SoMap();
		map.set("admin", admin);	
		map.set("perList", spRolePermissionService.getPcodeByRid(admin.getRoleId()));				
		map.set("appCfg", SpCfgUtil.getAppCfg());	
		return AjaxJson.getSuccessData(map); 
	}


}
