package com.pj.project.hc_camera;

import java.io.Serializable;
import java.util.Arrays;
import java.util.Date;

import com.alibaba.fastjson.annotation.JSONField;
import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonIgnore;
import lombok.AllArgsConstructor;
import lombok.EqualsAndHashCode;

import lombok.Data;
import lombok.Getter;
import lombok.experimental.Accessors;

/**
 * Model: hc_camera --
 *
 * @author qzyReal
 */
@Data
@Accessors(chain = true)
@TableName(HcCamera.TABLE_NAME)
@EqualsAndHashCode(callSuper = false)
public class HcCamera extends Model<HcCamera> implements Serializable {

    // ---------- 模块常量 ----------
    /**
     * 序列化版本id
     */
    private static final long serialVersionUID = 1L;
    /**
     * 此模块对应的表名
     */
    public static final String TABLE_NAME = "hc_camera";
    /**
     * 此模块对应的权限码
     */
    public static final String PERMISSION_CODE = "hc-camera";
    public static final String PERMISSION_CODE_LIST = "hc-camera-list";
    public static final String PERMISSION_CODE_ADD = "hc-camera-add";
    public static final String PERMISSION_CODE_EDIT = "hc-camera-edit";
    public static final String PERMISSION_CODE_DEL = "hc-camera-del";


    // ---------- 表中字段 ----------
    /**
     * 主键
     */
    @TableId(type = IdType.AUTO)
    private Long id;

    /**
     * 所在场所
     */
    private Long venuesId;

    /**
     * 名称
     */
    private String name;

    /**
     * IP地址
     */
    private String ip;


    private Integer status = 0;

    /**
     * 端口
     */
    private short port = 8000;

    /**
     * 用户名
     */
    private String uname;

    /**
     * 密码
     */
    @JsonIgnore
    private String pwd;

    /**
     * 设备唯一标识
     */
    private Integer userId=-1;
    /**
     * 通道名称
     */
    private String channelName;
    /**
     * 创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    private Date createTime;

    /**
     * 创建人
     */
    @TableField(fill = FieldFill.INSERT)
    private String createBy;

    /**
     * 更新时间
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private Date updateTime;

    /**
     * 更新人
     */
    @TableField(fill = FieldFill.INSERT_UPDATE)
    private String updateBy;

    @TableField(exist = false)
    private String venuesName;

    @TableField(exist = false)
    private String statusStr;

    @TableField(exist = false)
    private String flag;

    public String getFlag() {
        return "x"+getUserId();
    }

    public String getStatusStr() {
        return StatusEnum.getStr(this.getStatus());
    }

    @Getter
    @AllArgsConstructor
    public static enum StatusEnum {
        ONLINE(1, "在线"),
        OFF_LINE(0, "离线");
        private int status;
        private String statusStr;

        public static String getStr(int code) {
            return Arrays.stream(StatusEnum.values()).filter(s -> s.getStatus() == code).findAny().orElse(OFF_LINE).getStatusStr();
        }
    }

}
