package com.pj.project.sp_admin;

import cn.hutool.core.util.PhoneUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import com.pj.current.config.SystemObject;
import com.pj.current.global.BusinessException;
import com.pj.current.satoken.AuthConst;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_manager.TbManager;
import com.pj.project.tb_manager.TbManagerService;
import com.pj.project.tb_mini_user.TbMiniUser;
import com.pj.project.tb_mini_user.TbMiniUserService;
import com.pj.utils.sg.AjaxJson;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Propagation;
import org.springframework.transaction.annotation.Transactional;

import com.pj.project4sp.SP;

import cn.dev33.satoken.stp.StpUtil;

import javax.annotation.Resource;
import java.util.Date;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

/**
 * Service: admin管理员
 *
 * @author kong
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class SpAdminService {


    @Resource
    SpAdminMapper spAdminMapper;

    @Resource
    SpAdminPasswordService spAdminPasswordService;
    @Resource
    private TbDeptService tbDeptService;

    @Resource
    private TbMiniUserService tbMiniUserService;
    @Resource
    private TbManagerService tbManagerService;


    /**
     * 管理员添加一个管理员
     *
     * @param admin
     * @return
     */
    public long add(SpAdmin admin) {
        String phone = admin.getPhone();
        if (!PhoneUtil.isPhone(phone)) {
            throw new BusinessException("请输入正确的手机号");
        }
        // 检查姓名是否合法
        SpAdminUtil.checkAdmin(admin);

        // 创建人，为当前账号
        admin.setCreateByAid(StpUtil.getLoginIdAsLong());
        // 开始添加
        spAdminMapper.add(admin);
        // 获取主键
        long id = SP.publicMapper.getPrimarykey();
        // 更改密码（md5与明文）
        spAdminPasswordService.updatePassword(id, admin.getPassword2());

        // 返回主键
        return id;
    }


    /**
     * 绑定用户
     *
     * @param openid
     * @param key
     * @param password
     * @param type
     * @param name
     * @return
     */
    public AjaxJson bindUser(String openid, String key, String password, Integer type, String name, Long deptId) {
        TbMiniUser tbMiniUser = tbMiniUserService.findByUsername(key);
        if (tbMiniUser != null) {//直接登录
            if (tbMiniUser.getType().equals(type)) {
                String md5Password = SystemObject.getPasswordMd5(tbMiniUser.getSpAdminId(), password);
                if (!tbMiniUser.getPassword().equals(md5Password)) {
                    return AjaxJson.getError("账号/密码错误");
                }
                tbMiniUser.setOpenid(openid).setLastLoginTime(new Date());
                tbMiniUserService.updateById(tbMiniUser);
                return AjaxJson.getSuccessData(tbMiniUserService.buildToken(tbMiniUser));
            } else {
                return AjaxJson.getError("请更换入口");
            }
        }
        //绑定操作
        if (TbMiniUser.TypeEnum.JUDGE_TYPE.getType().equals(type)) {
            SpAdmin spAdmin;
            if (PhoneUtil.isPhone(key)) {
                spAdmin = spAdminMapper.getByPhone(key);
            } else {
                spAdmin = spAdminMapper.getByName(key);
            }
            if (spAdmin == null) {
                return AjaxJson.getError("账号/密码错误");
            }
            String md5Password = SystemObject.getPasswordMd5(spAdmin.getId(), password);
            if (!spAdmin.getPassword2().equals(md5Password)) {
                return AjaxJson.getError("账号/密码错误");
            }
            // 4、是否禁用
            if (spAdmin.getStatus() == 2) {
                return AjaxJson.getError("此账号已被禁用");
            }
            tbMiniUser = buildMiniUser(spAdmin, openid);
            tbMiniUser.setPassword(md5Password).setPhone(key).setUsername(key);
            tbMiniUserService.save(tbMiniUser);
            if (PhoneUtil.isPhone(key)) {
                List<TbManager> managers = tbManagerService.findByPhone(key);
                managers.forEach(tbManager -> tbManager.setOpenid(openid));
                tbManagerService.updateBatchById(managers);
            }
            return AjaxJson.getSuccessData(tbMiniUserService.buildToken(tbMiniUser));
        } else {
            tbMiniUser = createMiniUser(key, openid, name);
            String md5Password = SystemObject.getPasswordMd5(tbMiniUser.getSpAdminId(), password);
            TbDept tbDept=tbDeptService.getById(deptId);
            tbMiniUser.setPassword(md5Password).setDepartmentId(tbDept.getDepartmentId())
                    .setLastLoginTime(new Date())
                    .setDeptId(deptId);
            tbMiniUserService.updateById(tbMiniUser);
            return AjaxJson.getSuccessData(tbMiniUserService.buildToken(tbMiniUser));
        }
    }


    private TbMiniUser createMiniUser(String key, String openid, String name) {
        TbMiniUser tbMiniUser = new TbMiniUser();
        tbMiniUser.setCreateTime(new Date()).setSpAdminId(RandomUtil.randomLong(100000, 999999))
                .setOpenid(openid).setSuperAdmin(0)
                .setPhone(key).setType(TbMiniUser.TypeEnum.TEMPORARY_TYPE.getType())
                .setName(name).setUsername(key);
        tbMiniUserService.save(tbMiniUser);
        return tbMiniUser;
    }

    /**
     * 构建
     * @param spAdmin
     * @param openid
     * @return
     */
    private TbMiniUser buildMiniUser(SpAdmin spAdmin, String openid) {
        TbMiniUser tbMiniUser = new TbMiniUser();
        int superAdmin = StrUtil.equals(spAdmin.getDepartmentId(), AuthConst.SUPER_DEPARTMENT_ID) ? 1 : 0;
        tbMiniUser.setCreateTime(new Date()).setSuperAdmin(superAdmin)
                .setOpenid(openid).setSpAdminId(spAdmin.getId())
                .setPhone(spAdmin.getPhone()).setDepartmentId(spAdmin.getDepartmentId())
                .setType(TbMiniUser.TypeEnum.JUDGE_TYPE.getType())
                .setName(spAdmin.getNickname());
        return tbMiniUser;
    }

    public void updateDepartment(Long id, String departmentId) {
        spAdminMapper.updateDepartment(id, departmentId);
    }
}
