package com.pj.project.tb_car_filing;

import java.io.Serializable;
import java.util.Date;

import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import lombok.EqualsAndHashCode;

import lombok.Data;
import lombok.experimental.Accessors;

/**
 * Model: tb_car_filing -- 车辆备案
 * @author qzyReal 
 */
@Data
@Accessors(chain = true)
@TableName(TbCarFiling.TABLE_NAME)
@EqualsAndHashCode(callSuper = false)
public class TbCarFiling extends Model<TbCarFiling> implements Serializable {

	// ---------- 模块常量 ----------
	/**
	 * 序列化版本id 
	 */
	private static final long serialVersionUID = 1L;	
	/**
	 * 此模块对应的表名 
	 */
	public static final String TABLE_NAME = "tb_car_filing";	
	/**
	 * 此模块对应的权限码 
	 */
	public static final String PERMISSION_CODE_LIST = "tb-car-filing-list";
	public static final String PERMISSION_CODE_ADD = "tb-car-filing-add";
	public static final String PERMISSION_CODE_DEL = "tb-car-filing-del";
	public static final String PERMISSION_CODE_EDIT = "tb-car-filing-edit";
	public static final String PERMISSION_CODE_JUDGE = "tb-car-filing-judge";
	public static final String PERMISSION_CODE_EXPORT = "tb-car-filing-export";


	// ---------- 表中字段 ----------
	/**
	 *  
	 */
	@TableId(type = IdType.AUTO)
	@ExcelIgnore
	private Long id;
	@ExcelIgnore
	private String departmentId;
	/**
	 * 场所ID
	 */
	@ExcelIgnore
	private Long venuesId;

	/**
	 * 组织ID
	 */
	@ExcelIgnore
	private Long deptId;	


	/**
	 * 车牌号
	 */
	@ExcelProperty(index = 0,value = "车牌号")
	private String carNo;
	@ExcelProperty(index = 1,value = "备案组织")
	private String deptName;
	/**
	 * 类型(1=临时,2=内部)
	 */
	@ExcelIgnore
	private int type=1;

	/**
	 * 联系人
	 */
	@ExcelProperty(index = 2,value = "备案司机")
	private String driverName;
	/**
	 * 司机工号
	 */
	@ExcelIgnore
	private String driverPin;
	/**
	 * 司机人脸
	 */
	@ExcelIgnore
	private String driverPhoto;

	/**
	 * 联系电话 
	 */
	@ExcelProperty(index = 3,value = "联系号码")
	private String driverContact;	

	/**
	 * 联系人身份证 
	 */
	@ExcelIgnore
	private String driverIdCard;	

	/**
	 * 预计来访时间 
	 */
	@ExcelIgnore
	private Date preComeTime;

	/**
	 * 预计结束时间 
	 */
	@ExcelIgnore
	private Date preLeaveTime;

	/**
	 * 状态 0未到访，1已来防
	 */
	@ExcelIgnore
	private int visit;
	@ExcelProperty(index =7,value = "商铺号")
	private String shopCode;
	/**
	 * 备注 
	 */
	@ExcelIgnore
	private String remark;

	/**
	 * 备案过期时间
	 */
	@ExcelProperty(index =8,value = "有效期")
	private Date endTime;

	/**
	 * 创建时间 
	 */
	@TableField(fill = FieldFill.INSERT)
	@ExcelProperty(index =9,value = "创建时间")
	private Date createTime;

	/**
	 * 创建人 
	 */
	@TableField(fill = FieldFill.INSERT)
	@ExcelIgnore
	private String createBy;	

	/**
	 * 更新时间 
	 */
	@TableField(fill = FieldFill.UPDATE)
	@ExcelIgnore
	private Date updateTime;

	/**
	 * 更新人 
	 */
	@TableField(fill = FieldFill.UPDATE)
	@ExcelIgnore
	private String updateBy;
	@ExcelIgnore
	private Integer judgeState=0;
	@ExcelProperty(index =10,value = "审核时间")
	private Date judgeTime;
	@ExcelProperty(index =11,value = "审核人")
	private String judgeBy;
	/**
	 * 审核意见
	 */
	@ExcelProperty(index =12,value = "审核意见")
	private String judgeContent;
	@TableField(exist = false)
	@ExcelProperty(index =6,value = "审核部门")
	private String departmentName;

	@TableField(exist = false)
	@ExcelProperty(index =5,value = "备案场所")
	private String venuesName;

	@TableField(exist = false)
	@ExcelProperty(index =4,value = "司机身份证")
	private String driverIdCardStr;
	@ExcelIgnore
	@TableField(exist = false)
	private String note;

	public String getNote() {
		String name=this.getDriverName();
		String phone=this.getDriverContact();
		if (StrUtil.isNotEmpty(phone)){
			return name+"("+phone+")";
		}
		return name;
	}

	public String getDriverIdCardStr() {
		String driverIdCard=getDriverIdCard();
		if (StrUtil.isEmpty(driverIdCard)||!IdcardUtil.isValidCard(driverIdCard)){
			return driverIdCard;
		}
		String str = IdcardUtil.getBirthByIdCard(driverIdCard);
		if (StrUtil.isNotEmpty(str)) {
			return driverIdCard.replace(str, "********");
		}
		return driverIdCard;
	}
}
