package com.pj.project.tb_car_filing;

import java.io.File;
import java.util.Date;
import java.util.List;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.codec.Base64Encoder;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.config.SystemObject;
import com.pj.current.config.WxConfig;
import com.pj.current.global.BusinessException;
import com.pj.current.netty.command.constants.CommandType;
import com.pj.current.netty.command.handler.dto.WhitelistDTO;
import com.pj.current.satoken.StpUserUtil;
import com.pj.project.re_mini_user_person_filing.ReMiniUserPersonFiling;
import com.pj.project.re_mini_user_person_filing.ReMiniUserPersonFilingService;
import com.pj.project.tb_car_black.TbCarBlack;
import com.pj.project.tb_car_black.TbCarBlackService;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_mini_user.TbMiniUser;
import com.pj.project.tb_mini_user.TbMiniUserService;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_filing.TbPersonFilingService;
import com.pj.project.tb_terminal.TbTerminal;
import com.pj.project.tb_terminal.TbTerminalService;
import com.pj.project.tb_venues.TbVenues;
import com.pj.project.tb_venues.TbVenuesService;
import com.pj.project.wx.WxService;
import com.pj.project.wx.bo.MsgDataBO;
import com.pj.project4sp.spcfg.SpCfgUtil;
import com.pj.project4sp.uploadfile.UploadConfig;
import com.pj.utils.sg.AjaxError;
import lombok.extern.slf4j.Slf4j;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_car_filing -- 车辆备案
 *
 * @author qzyReal
 */
@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class TbCarFilingService extends ServiceImpl<TbCarFilingMapper, TbCarFiling> implements IService<TbCarFiling> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbCarFilingMapper tbCarFilingMapper;

    @Resource
    private TbDeptService tbDeptService;
    @Resource
    private TbPersonFilingService tbPersonFilingService;
    @Resource
    private WxConfig wxConfig;
    @Resource
    private WxService wxService;
    @Resource
    private TbMiniUserService tbMiniUserService;
    @Resource
    private UploadConfig uploadConfig;

    /**
     * 增
     */
    public void add(TbCarFiling t) {
        String carNo = t.getCarNo().toUpperCase();
        if (StrUtil.isEmpty(carNo)) {
            throw new BusinessException("请输入车牌号");
        }
        String idCarNo = t.getDriverIdCard();
        Long deptId = t.getDeptId();
        if (deptId == null) {
            throw new BusinessException("请选择备案组织");
        }
        TbDept tbDept = tbDeptService.getById(deptId);
        if (tbDept.getPassType().equals(TbDept.PassTypeEnum.BLOW.getType())) {
            if (!IdcardUtil.isValidCard(idCarNo)) {
                throw new BusinessException("身份证不正确");
            }
        }
        if (TbDept.NeedJudgeEnum.AUTO_JUDGE.getType().equals(tbDept.getNeedJudge())) {
            t.setJudgeState(1).setJudgeBy("系统自动审核").setJudgeTime(new Date());
        }
        TbCarFiling check = findByCarNoAndIdCard(carNo, idCarNo);
        if (check != null) {
            throw new AjaxError("该司机已备案该车辆");
        }
        t.setDeptName(tbDept.getName()).setCarNo(carNo).setDepartmentId(tbDept.getDepartmentId())
                .setVenuesId(tbDept.getVenuesId());
        //原来是否录入过
        List<TbCarFiling> carFilingList = this.findByDriverIdCar(idCarNo);
        carFilingList.forEach(tbCarFiling -> {
            if (tbCarFiling != null && StrUtil.equals(tbCarFiling.getCarNo(), carNo)) {
                throw new BusinessException("司机已备案该车辆");
            }
        });
        handlerDriver(t);
        this.save(t);
    }

    /**
     * 处理备案司机
     *
     * @param tbCarFiling
     */
    private void handlerDriver(TbCarFiling tbCarFiling) {
        String idCarNo = tbCarFiling.getDriverIdCard();
        if (StrUtil.isEmpty(idCarNo) || !IdcardUtil.isValidCard(idCarNo)) {
            return;
        }
        //人员是否备案
        TbPersonFiling t = tbPersonFilingService.findByIdCard(idCarNo);
        Long deptId = tbCarFiling.getDeptId();
        TbDept tbDept = tbDeptService.getById(deptId);
        tbCarFiling.setDeptName(tbDept.getName());
        boolean judge = TbDept.NeedJudgeEnum.AUTO_JUDGE.getType().equals(tbDept.getNeedJudge());
        if (judge) {
            tbCarFiling.setJudgeState(1).setJudgeTime(new Date()).setJudgeBy("系统自动审核");
        }
        if (t == null) {
            String pin = tbCarFiling.getDriverIdCard();
            tbCarFiling.setDriverPin(pin);
            t = new TbPersonFiling();
            t.setDeptId(tbDept.getId())
                    .setPhoto(tbCarFiling.getDriverPhoto())
                    .setDeptName(tbDept.getName())
                    .setType(tbCarFiling.getType())
                    .setPin(idCarNo).setDepartmentId(tbCarFiling.getDepartmentId())
                    .setVenuesId(tbCarFiling.getVenuesId())
                    .setIdCard(tbCarFiling.getDriverIdCard())
                    .setName(tbCarFiling.getDriverName())
                    .setPhone(tbCarFiling.getDriverContact());
            if (judge) {
                t.setJudgeState(1).setJudgeTime(new Date()).setJudgeBy("系统自动审核");
                // todo 下发人脸到设备
                tbPersonFilingService.issued(t);
            }
            tbPersonFilingService.save(t);
        }
    }

    /**
     * 一个司机只能备案一次
     *
     * @param idCarNo
     * @return
     */
    public List<TbCarFiling> findByDriverIdCar(String idCarNo) {
        QueryWrapper<TbCarFiling> ew = new QueryWrapper<>();
        ew.eq("driver_id_card", idCarNo);
        return list(ew);
    }


    /**
     * 删
     */
    public void delete(Long id) {
        TbCarFiling db = this.getById(id);
        String idCarNo = db.getDriverIdCard();
        TbPersonFiling tbPersonFiling = tbPersonFilingService.findByIdCard(idCarNo);
        //如果不存在白名单，则
        if (tbPersonFiling == null) {
            //删除设备的白名单
            tbPersonFilingService.deleteTerminalWhilte(db.getDriverPin(), true);
        }
        this.removeById(id);
    }

    /**
     * 根据车牌号和身份证查找记录
     *
     * @param carNo
     * @param idCard
     * @return
     */
    public TbCarFiling findByCarNoAndIdCard(String carNo, String idCard) {
        QueryWrapper<TbCarFiling> ew = new QueryWrapper<>();
        ew.lambda().eq(TbCarFiling::getCarNo, carNo.toUpperCase())
                .eq(TbCarFiling::getDriverIdCard, idCard);
        return getOne(ew);
    }

    /**
     * 改
     */
    public void update(TbCarFiling t) {
        String carNo = t.getCarNo().toUpperCase();
        String idCarNo = t.getDriverIdCard();
        TbCarFiling db = this.getById(t.getId());
        TbCarFiling check = findByCarNoAndIdCard(carNo, idCarNo);
        if (check != null && !db.getId().equals(check.getId())) {
            throw new AjaxError("该司机已备案该车辆");
        }
        if (!t.getDeptId().equals(db.getDeptId())) {
            TbDept tbDept = tbDeptService.getById(t.getDeptId());
            t.setDeptName(tbDept.getName());
        }

        String oldIdCardNo = db.getDriverIdCard();
        //改了身份证或者人脸或者姓名
        if (!StrUtil.equals(idCarNo, oldIdCardNo)
                || !StrUtil.equals(t.getDriverPhoto(), db.getDriverPhoto())
                || !StrUtil.equals(t.getDriverName(), db.getDriverName())
        ) {
            //更改备案人信息
            TbPersonFiling tbPersonFiling = tbPersonFilingService.findByIdCard(oldIdCardNo);
            if (tbPersonFiling != null) {
                tbPersonFiling.setIdCard(idCarNo)
                        .setName(t.getDriverName()).setDepartmentId(t.getDepartmentId())
                        .setVenuesId(t.getVenuesId())
                        .setPhone(t.getDriverContact());
                tbPersonFilingService.updateById(tbPersonFiling);
            }
            //司机身份证不一样 删除旧的，下发新的
            tbPersonFilingService.deleteTerminalWhilte(db.getDriverIdCard(), false);
            tbPersonFilingService.issued(idCarNo);
        }
        t.setCarNo(carNo);
        this.updateById(t);
    }

    /**
     * 查
     */
    public TbCarFiling getById(Long id) {
        return tbCarFilingMapper.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbCarFiling> getList(SoMap so) {
        return tbCarFilingMapper.getList(so);
    }


    /**
     * 审核
     *
     * @param t
     */
    public void judge(TbCarFiling t) {
        TbCarFiling db = this.getById(t.getId());
        int judgeState = t.getJudgeState();
        db.setJudgeTime(new Date()).setJudgeBy(StpUserUtil.getAdminName()).setJudgeState(judgeState);
        this.updateById(db);
        //人员备案审核
        tbPersonFilingService.judgeByIdCard(db.getDriverIdCard(), db.getJudgeState());
        List<TbMiniUser> miniUsers = tbMiniUserService.findByCarFilingId(t.getId());
        miniUsers.parallelStream().forEach(tbMiniUser -> {
            MsgDataBO data = new MsgDataBO("您有一个备案审核提醒", judgeState == 2 ? "不通过" : "通过"
                    , tbMiniUser.getName(), t.getCarNo() + "车辆备案", "审核时间：" + DateUtil.now());
            wxService.sendTemplateMsg(wxConfig.getJudgeResultTemplate(), tbMiniUser.getOpenid(), data);
        });
    }


    public List<TbCarFiling> getOuterList(SoMap soMap) {
        return tbCarFilingMapper.getOuterList(soMap);
    }


    public List<TbCarFiling> findByCarNo(String carNo) {
        QueryWrapper<TbCarFiling> ew = new QueryWrapper<>();
        ew.lambda().eq(TbCarFiling::getCarNo, carNo);
        return list(ew);
    }

    public String export(SoMap so) {
        List<TbCarFiling>list=tbCarFilingMapper.getList(so);
        String flieTypeFolder = "/export/";
        String currDateFolder = DateUtil.today();
        String fileName = "car-filing-" + RandomUtil.randomNumbers(6) + ".xlsx";
        String fileFolder = new File(uploadConfig.rootFolder).getAbsolutePath() +
                uploadConfig.httpPrefix + flieTypeFolder + currDateFolder + "/";
        if (!FileUtil.exist(fileFolder)) {
            FileUtil.mkdir(fileFolder);
        }
        EasyExcel.write(fileFolder + fileName, TbCarFiling.class).sheet("车辆备案信息")
                .doWrite(() -> list);
        return SystemObject.config.getDomain() + uploadConfig.httpPrefix + flieTypeFolder + currDateFolder + "/" + fileName;
    }

    /**
     * 需要审核的数量
     * @param departmentId
     * @return
     */
    public int getNeedJudgeNum(String departmentId) {
        QueryWrapper<TbCarFiling>ew=new QueryWrapper<>();
        ew.lambda().eq(StrUtil.isNotEmpty(departmentId),TbCarFiling::getDepartmentId,departmentId)
                .eq(TbCarFiling::getJudgeState,0);
        return this.count(ew);
    }
}
