package com.pj.project.tb_car_visit_record;

import java.io.File;
import java.util.Date;
import java.util.List;

import cn.hutool.core.date.DateUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.poi.excel.ExcelUtil;
import cn.hutool.poi.excel.ExcelWriter;
import com.alibaba.excel.EasyExcel;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.config.SystemObject;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_person_filing.TbPersonFiling;
import com.pj.project.tb_person_visit_record.TbPersonVisitRecord;
import com.pj.project.tb_venues.TbVenues;
import com.pj.project.tb_venues.TbVenuesService;
import com.pj.project4sp.uploadfile.UploadConfig;
import com.pj.utils.sg.AjaxError;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_car_visit_record -- 车辆出入记录
 *
 * @author qzyReal
 */
@Service
@Transactional(rollbackFor = Exception.class)
public class TbCarVisitRecordService extends ServiceImpl<TbCarVisitRecordMapper, TbCarVisitRecord> implements IService<TbCarVisitRecord> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbCarVisitRecordMapper tbCarVisitRecordMapper;
    @Resource
    private TbVenuesService tbVenuesService;

    @Resource
    private UploadConfig uploadConfig;
    @Resource
    private TbDeptService tbDeptService;

    /**
     * 增
     */
    public void add(TbCarVisitRecord t) {
       Long deptId= t.getDeptId();
       if (deptId==null){
           throw new AjaxError("请选择组织");
       }
        if (t.getChannelId()==null){
            throw new AjaxError("请选择通道");
        }
        TbDept tbDept=tbDeptService.getById(deptId);
        TbVenues channel = tbVenuesService.getById(t.getChannelId());
        t.setRecordType(1).setCarNo(t.getCarNo().toUpperCase()).setVenuesId(channel.getPId())
                .setChannel(channel.getName()).setDeptName(tbDept.getName())
                .setDepartmentId(tbDept.getDepartmentId());

        this.save(t);
    }

    /**
     * 删
     */
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbCarVisitRecord t) {
        TbVenues tbVenues = tbVenuesService.getById(t.getChannelId());
        t.setCarNo(t.getCarNo().toUpperCase())
                .setChannel(tbVenues.getName());
        this.updateById(t);
    }

    /**
     * 查
     */
    public TbCarVisitRecord getById(Long id) {
        return super.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbCarVisitRecord> getList(SoMap so) {
        return tbCarVisitRecordMapper.getList(so);
    }


    public String export(SoMap soMap) {
        List<TbCarVisitRecord> list = this.getList(soMap);
        String flieTypeFolder = "/export/";
        String currDateFolder = DateUtil.today();
        String fileName = "car-record-" + RandomUtil.randomNumbers(6) + ".xlsx";
        String fileFolder = new File(uploadConfig.rootFolder).getAbsolutePath() +
                uploadConfig.httpPrefix + flieTypeFolder + currDateFolder + "/";
        if (!FileUtil.exist(fileFolder)) {
            FileUtil.mkdir(fileFolder);
        }
        EasyExcel.write(fileFolder + fileName, TbCarVisitRecord.class).sheet("车辆出入记录")
                .doWrite(() -> list);
        return SystemObject.config.getDomain() + uploadConfig.httpPrefix + flieTypeFolder + currDateFolder + "/" + fileName;
    }

    public void updatePass(Long recordId) {
       TbCarVisitRecord record= getById(recordId);
       record.setAllowPass(1);
       this.updateById(record);
    }
}
