package com.pj.project.tb_mini_user;

import java.util.*;

import cn.dev33.satoken.stp.StpUtil;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.config.SystemObject;
import com.pj.current.satoken.AuthConst;
import com.pj.current.satoken.StpUserUtil;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.utils.sg.AjaxJson;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;

import javax.annotation.Resource;

/**
 * Service: tb_mini_user -- h5端用户
 *
 * @author qzyReal
 */
@Service
public class TbMiniUserService extends ServiceImpl<TbMiniUserMapper, TbMiniUser> implements IService<TbMiniUser> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbMiniUserMapper tbMiniUserMapper;
    @Resource
    private TbDeptService tbDeptService;


    /**
     * 增
     */
    public void add(TbMiniUser t) {
        this.save(t);
    }

    /**
     * 删
     */
    public void delete(Long id) {
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbMiniUser t) {
        this.updateById(t);
    }

    /**
     * 查
     */
    public TbMiniUser getById(Long id) {
        return super.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbMiniUser> getList(SoMap so) {
        return tbMiniUserMapper.getList(so);
    }

    /**
     * 根据open和type查找
     *
     * @param openid
     * @param type
     * @return
     */
    public TbMiniUser findByOpenid(String openid, Integer type) {
        QueryWrapper<TbMiniUser> ew = new QueryWrapper<>();
        ew.lambda().eq(TbMiniUser::getOpenid, openid)
                .eq(TbMiniUser::getType, type)
                .orderByDesc(TbMiniUser::getLastLoginTime);
        List<TbMiniUser> list = list(ew);
        return list.isEmpty() ? null : list.get(0);
    }


    public void removeByPhone(String phone) {
        QueryWrapper<TbMiniUser> ew = new QueryWrapper<>();
        ew.lambda().eq(TbMiniUser::getPhone, phone);
        this.remove(ew);
    }

    /**
     * openid登录
     *
     * @param openid
     * @param type
     * @return
     */
    public AjaxJson loginByOpenid(String openid, Integer type) {
        TbMiniUser tbMiniUser = this.findByOpenid(openid, type);
        if (tbMiniUser == null) {
            return AjaxJson.getError(AjaxJson.ERROR_ENTER_PORT, "请更换入口");
        }
        tbMiniUser.setLastLoginTime(new Date());
        this.updateById(tbMiniUser);
        return AjaxJson.getSuccessData(buildToken(tbMiniUser));
    }

    public TbMiniUser findByPhone(String phone) {
        QueryWrapper<TbMiniUser> ew = new QueryWrapper<>();
        ew.lambda().eq(TbMiniUser::getPhone, phone);
        return getOne(ew);
    }


    public AjaxJson login(String openid, String key, String password) {
        TbMiniUser tbMiniUser = this.findByPhone(key);
        if (tbMiniUser == null) {
            return AjaxJson.getError("账户不存在");
        }
        String md5Password = SystemObject.getPasswordMd5(tbMiniUser.getSpAdminId(), password);
        if (!tbMiniUser.getPassword().equals(md5Password)) {
            return AjaxJson.getError("密码错误");
        }
        tbMiniUser.setOpenid(openid);
        this.updateById(tbMiniUser);
        return AjaxJson.getSuccessData(buildToken(tbMiniUser));
    }

    /**
     * 构建token
     *
     * @param tbMiniUser
     * @return
     */
    public Map<String, Object> buildToken(TbMiniUser tbMiniUser) {
        StpUtil.login(tbMiniUser.getId());
        StpUserUtil.setAdmin(tbMiniUser.getName(), tbMiniUser.getPhone());
        StpUserUtil.setMiniUserId(tbMiniUser.getId());
        StpUserUtil.setDepartmentId(tbMiniUser.getDepartmentId());
        StpUserUtil.setDeptIdList(Collections.singletonList(tbMiniUser.getDeptId()));
        StpUserUtil.setMiniUserType(tbMiniUser.getType());
        String token = StpUtil.getTokenValue();
        tbMiniUser.setToken(token);
        Map<String, Object> map = new HashMap<>();
        TbDept tbDept = tbDeptService.getById(tbMiniUser.getDeptId());
        if (tbDept != null) {
            tbMiniUser.setDeptName(tbDept.getName());
        }
        map.put("userInfo", tbMiniUser);
        map.put("token", token);
        return map;
    }

    public TbMiniUser findByUsername(String username) {
        QueryWrapper<TbMiniUser> ew = new QueryWrapper<>();
        ew.lambda().eq(TbMiniUser::getUsername, username);
        return getOne(ew);
    }

    public void updateDept(Long id, Long deptId) {
        TbMiniUser tbMiniUser = this.getById(id);
        TbDept tbDept = tbDeptService.getById(deptId);
        tbMiniUser.setDeptId(deptId).setDeptName(tbDept.getName()).setDepartmentId(tbDept.getDepartmentId());
        this.updateById(tbMiniUser);
    }

    /**
     * 更新密码
     *
     * @param id
     * @param password
     */
    public void updatePassword(Long id, String password) {
        TbMiniUser tbMiniUser = this.getById(id);
        String md5Password = SystemObject.getPasswordMd5(tbMiniUser.getSpAdminId(), password);
        tbMiniUser.setPassword(md5Password);
        this.updateById(tbMiniUser);
    }

    public void removeBySpAdmin(long adminId) {
        QueryWrapper<TbMiniUser>ew=new QueryWrapper<>();
        ew.lambda().eq(TbMiniUser::getSpAdminId,adminId);
        this.remove(ew);
    }

    /**
     * 根据备案ID查找
     * @param filingId
     * @return
     */
    public List<TbMiniUser>findByPersonFilingId(Long filingId){
        return tbMiniUserMapper.findByPersonFilingId(filingId);
    }
    public List<TbMiniUser>findByCarFilingId(Long filingId){
        return tbMiniUserMapper.findByCarFilingId(filingId);
    }

    public List<TbMiniUser> findAdminList() {
        QueryWrapper<TbMiniUser>ew=new QueryWrapper<>();
        ew.lambda().eq(TbMiniUser::getDepartmentId, AuthConst.SUPER_DEPARTMENT_ID);
        return list(ew);
    }
}
