package com.pj.project.tb_person_filing;

import java.io.Serializable;
import java.util.Date;

import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.StrUtil;
import com.alibaba.excel.annotation.ExcelIgnore;
import com.alibaba.excel.annotation.ExcelProperty;
import com.baomidou.mybatisplus.annotation.*;
import com.baomidou.mybatisplus.extension.activerecord.Model;
import com.fasterxml.jackson.annotation.JsonFormat;
import lombok.AllArgsConstructor;
import lombok.EqualsAndHashCode;

import lombok.Data;
import lombok.Getter;
import lombok.experimental.Accessors;
import org.springframework.format.annotation.DateTimeFormat;

/**
 * Model: tb_person_filing -- 人员备案
 *
 * @author qzyReal
 */
@Data
@Accessors(chain = true)
@TableName(TbPersonFiling.TABLE_NAME)
@EqualsAndHashCode(callSuper = false)
public class TbPersonFiling extends Model<TbPersonFiling> implements Serializable {

    // ---------- 模块常量 ----------
    /**
     * 序列化版本id
     */
    private static final long serialVersionUID = 1L;
    /**
     * 此模块对应的表名
     */
    public static final String TABLE_NAME = "tb_person_filing";
    /**
     * 此模块对应的权限码
     */
    public static final String PERMISSION_CODE_LIST = "tb-person-filing-list";
    public static final String PERMISSION_CODE_ADD = "tb-person-filing-add";
    public static final String PERMISSION_CODE_EDIT = "tb-person-filing-edit";
    public static final String PERMISSION_CODE_DEL = "tb-person-filing-del";
    public static final String PERMISSION_CODE_JUDGE = "tb-person-filing-judge";
    public static final String PERMISSION_CODE_ISSUED = "tb-person-filing-issued";
    public static final String PERMISSION_CODE_IMPORT = "tb-person-filing-import";
    public static final String PERMISSION_CODE_EXPORT = "tb-person-filing-export";


    // ---------- 表中字段 ----------
    /**
     *
     */
    @TableId(type = IdType.AUTO)
    @ExcelIgnore
    private Long id;
    /**
     * 场所ID
     */
    @ExcelIgnore
    private Long venuesId;
    /**
     * 部门ID
     */
    @ExcelIgnore
    private String departmentId;
    /**
     * 组织ID
     */
    @ExcelIgnore
    private Long deptId;

    /**
     * 姓名
     */
    @ExcelProperty(index = 0,value = "姓名")
    private String name;
    /**
     * 工号
     */
    @ExcelIgnore
    private String pin;
    @ExcelProperty(index = 1,value = "手机号")
    private String phone;


    /**
     * 类型(1=临时,2=内部)
     */
    @ExcelIgnore
    private Integer type = 1;

    /**
     * 身份证
     */
    @ExcelIgnore
    private String idCard;
    /**
     * 身份证图片url
     */
    @ExcelIgnore
    private String idCardUrl;

    /**
     * 来访事由
     */
    @ExcelIgnore
    private String visitReason;

    /**
     * 预计来访时间
     */
    @ExcelIgnore
    private Date preComeTime;

    /**
     * 预计结束时间
     */
    @ExcelIgnore
    private Date preLeaveTime;

    /**
     * 状态
     */
    @ExcelIgnore
    private String visit;
    @ExcelIgnore
    private String photo;

    /**
     * 备注
     */
    @ExcelIgnore
    private String remark;

    /**
     * 创建时间
     */
    @TableField(fill = FieldFill.INSERT)
    @ExcelProperty(index = 6,value = "申请时间")
    private Date createTime;

    /**
     * 创建人
     */
    @TableField(fill = FieldFill.INSERT)
    @ExcelIgnore
    private String createBy;

    /**
     * 更新时间
     */
    @TableField(fill = FieldFill.UPDATE)
    @ExcelIgnore
    private Date updateTime;

    /**
     * 更新人
     */
    @TableField(fill = FieldFill.UPDATE)
    @ExcelIgnore
    private String updateBy;
    @ExcelIgnore
    private int judgeState;
    @ExcelProperty(index = 8,value = "审核时间")
    private Date judgeTime;
    @ExcelProperty(index = 7,value = "审核人")
    private String judgeBy;
    @ExcelIgnore
    private String openid;
    /**
     * 边民证号
     */
    @ExcelProperty(value = "边民证")
    private String icCard;
    /**
     * 边民证图片url
     */
    @ExcelIgnore
    private String icCardUrl;
    /**
     * 边民证或身份证图片url
     */
    @ExcelIgnore
    private String cardUrl;
    /**
     * 边民证有效期，起始日期
     */

    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @ExcelIgnore
    private Date icCardStartTime;
    /**
     * 边民证有效期，到期时间
     */
    @JsonFormat(pattern = "yyyy-MM-dd", timezone = "GMT+8")
    @DateTimeFormat(pattern = "yyyy-MM-dd")
    @ExcelProperty(index=9,value = "边民证有效期")
    private Date icCardEndTime;
    @ExcelProperty(index=10,value = "商铺号")
    private String shopCode;
    @ExcelProperty(index=11,value = "备案有效期")
    private Date endTime;
    /**
     * 审核意见
     */
    @ExcelProperty(value = "审核意见")
    private String judgeContent;


    @TableField(exist = false)
    @ExcelProperty(index = 4,value = "备案组织")
    private String deptName;

    @TableField(exist = false)
    @ExcelProperty(index = 2,value = "身份证")
    private String idCadStr;
    @TableField(exist = false)
    @ExcelProperty(index = 3,value = "备案场所")
    private String venuesName;

    @TableField(exist = false)
    @ExcelProperty(index = 5,value = "审核单位")
    private String departmentName;

    public String getIdCadStr() {
        String str = IdcardUtil.getBirthByIdCard(idCard);
        if (StrUtil.isNotEmpty(str)) {
            return idCard.replace(str, "********");
        }
        return idCard;
    }
    @Getter
    @AllArgsConstructor
    public static enum JudgeStatsEnum{
        PASS_STATE(1,"审核通过"),
        NOT_JUDGE_STATE(0,"未审核"),
        NOT_PASS_STATE(1,"审核驳回");
        private int code;
        private String desc;
    }
}
