package com.pj.project.tb_person_filing;

import java.io.File;
import java.io.InputStream;
import java.nio.charset.Charset;
import java.util.ArrayList;
import java.util.Collections;
import java.util.Date;
import java.util.List;
import java.util.stream.Collectors;

import cn.dev33.satoken.stp.StpUtil;
import cn.hutool.core.codec.Base64;
import cn.hutool.core.codec.Base64Encoder;
import cn.hutool.core.date.DateUtil;
import cn.hutool.core.img.ImgUtil;
import cn.hutool.core.io.FileUtil;
import cn.hutool.core.util.IdcardUtil;
import cn.hutool.core.util.RandomUtil;
import cn.hutool.core.util.StrUtil;
import cn.hutool.core.util.ZipUtil;
import cn.hutool.json.JSONArray;
import cn.hutool.json.JSONObject;
import cn.hutool.json.JSONUtil;
import cn.hutool.log.StaticLog;
import com.alibaba.excel.EasyExcel;
import com.alibaba.excel.ExcelReader;
import com.alibaba.excel.read.metadata.ReadSheet;
import com.baomidou.mybatisplus.core.conditions.query.QueryWrapper;
import com.baomidou.mybatisplus.extension.service.IService;
import com.baomidou.mybatisplus.extension.service.impl.ServiceImpl;
import com.pj.current.config.SystemObject;
import com.pj.current.config.WxConfig;
import com.pj.current.global.BusinessException;
import com.pj.current.netty.command.constants.CommandType;
import com.pj.current.netty.command.handler.dto.WhitelistDTO;
import com.pj.current.satoken.StpUserUtil;
import com.pj.current.task.TaskService;
import com.pj.project.tb_car_filing.TbCarFiling;
import com.pj.project.tb_car_filing.TbCarFilingService;
import com.pj.project.tb_department.TbDepartment;
import com.pj.project.tb_department.TbDepartmentService;
import com.pj.project.tb_dept.TbDept;
import com.pj.project.tb_dept.TbDeptService;
import com.pj.project.tb_mini_user.TbMiniUser;
import com.pj.project.tb_mini_user.TbMiniUserService;
import com.pj.project.tb_person_black.TbPersonBlack;
import com.pj.project.tb_person_filing.task.DelayIsuuedList;
import com.pj.project.tb_terminal.TbTerminal;
import com.pj.project.tb_terminal.TbTerminalService;
import com.pj.project.tb_venues.TbVenues;
import com.pj.project.tb_venues.TbVenuesService;
import com.pj.project.wx.WxService;
import com.pj.project.wx.bo.MsgDataBO;
import com.pj.project4sp.spcfg.SpCfgUtil;
import com.pj.project4sp.uploadfile.UploadConfig;
import com.pj.project4sp.uploadfile.UploadUtil;
import com.pj.utils.sg.AjaxError;
import com.sun.imageio.plugins.common.ImageUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.util.ImageUtils;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.context.annotation.Lazy;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;
import com.pj.utils.so.*;
import org.springframework.web.multipart.MultipartFile;

import javax.annotation.Resource;

/**
 * Service: tb_person_filing -- 人员备案
 *
 * @author qzyReal
 */
@Service
@Transactional(rollbackFor = Exception.class)
@Slf4j
public class TbPersonFilingService extends ServiceImpl<TbPersonFilingMapper, TbPersonFiling> implements IService<TbPersonFiling> {

    /**
     * 底层 Mapper 对象
     */
    @Autowired
    TbPersonFilingMapper tbPersonFilingMapper;
    @Resource
    @Lazy
    private TbTerminalService tbTerminalService;

    @Resource
    private UploadConfig uploadConfig;
    @Resource
    private TbDeptService tbDeptService;
    @Resource
    private ImportPersonListener importPersonListener;

    @Resource
    private TaskService taskService;
    @Resource
    @Lazy
    private TbCarFilingService tbCarFilingService;

    @Resource
    private TbVenuesService tbVenuesService;

    @Resource
    private WxConfig wxConfig;
    @Resource
    private WxService wxService;
    @Resource
    private TbMiniUserService tbMiniUserService;

    /**
     * 增
     */
    public void add(TbPersonFiling t) {
        String idCard = t.getIdCard();
        if (!IdcardUtil.isValidCard(idCard)) {
            throw new BusinessException("身份证不正确");
        }
        TbPersonFiling db = findByIdCard(idCard);
        if (db != null) {
            throw new BusinessException("人员已存在");
        }
        t.setPin(idCard);
        Long deptId = t.getDeptId();
        if (deptId == null) {
            throw new BusinessException("请选择备案组织");
        }
        TbDept tbDept = tbDeptService.getById(t.getDeptId());
        if (TbDept.NeedJudgeEnum.AUTO_JUDGE.getType().equals(tbDept.getNeedJudge())) {
            t.setJudgeState(1).setJudgeTime(new Date()).setJudgeBy("系统自动审核");
        }
        t.setVenuesId(tbDept.getVenuesId())
                .setDepartmentId(tbDept.getDepartmentId());
        this.save(t);
        issued(t);
    }


    /**
     * 删
     */
    public void delete(Integer id) {
        TbPersonFiling db = this.getById(id);
        List<TbCarFiling> tbCarFilingList = tbCarFilingService.findByDriverIdCar(db.getIdCard());
        tbCarFilingList.forEach(tbCarFiling -> {
            if (tbCarFiling.getJudgeState() == 0) {//车辆司机不存在或者未审核，删除备案
                deleteTerminalWhilte(db.getPin(), true);
            }
        });
        this.deleteTerminalWhilte(db.getPin(), true);
        this.removeById(id);
    }

    /**
     * 改
     */
    public void update(TbPersonFiling t) {
        String idCard = t.getIdCard();
        if (!IdcardUtil.isValidCard(idCard)) {
            throw new BusinessException("身份证不正确");
        }
        TbPersonFiling idCardDb = findByIdCard(idCard);
        if (idCardDb != null && !idCardDb.getId().equals(t.getId())) {
            throw new BusinessException("人员已存在");
        }
        Long deptId = t.getDeptId();
        if (deptId == null) {
            throw new BusinessException("请选择备案组织");
        }
        TbDept tbDept = tbDeptService.getById(t.getDeptId());
        TbPersonFiling db = this.getById(t.getId());
        // 改了备案组织、人脸、姓名、身份证都要重新下发
        if (!StrUtil.equals(t.getIdCard(), db.getIdCard())
                || !StrUtil.equals(t.getName(), db.getName())
                || !db.getVenuesId().equals(t.getVenuesId())) {
            //删除旧的
            deleteTerminalWhilte(db.getIdCard(), false);
            //下发新的
            issued(t);
        }
        t.setDepartmentId(tbDept.getDepartmentId()).setVenuesId(tbDept.getVenuesId());
        //重新上传了人脸
        if (!StrUtil.equals(t.getPhoto(), db.getPhoto())) {
            //下发新的
            issued(t);
        }

        this.updateById(t);
    }

    /**
     * 查
     */
    public TbPersonFiling getById(Integer id) {
        return tbPersonFilingMapper.getById(id);
    }

    /**
     * 查集合 - 根据条件（参数为空时代表忽略指定条件）
     */
    public List<TbPersonFiling> getList(SoMap so) {
        return tbPersonFilingMapper.getList(so);
    }

    /**
     * 根据身份证查找
     *
     * @param idCard
     * @return
     */
    public TbPersonFiling findByIdCard(String idCard) {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.eq("id_card", idCard);
        return getOne(ew);
    }

    /**
     * 下发单个白名单到设备
     *
     * @param filing
     */
    public void issued(TbPersonFiling filing) {
        if (filing.getJudgeState() != 1) {
            log.error("人员白名单未审核通过，不下发:{}", JSONUtil.toJsonStr(filing));
            return;
        }
        if (StrUtil.isEmpty(filing.getPhoto()) || StrUtil.isEmpty(filing.getIdCard())) {
            log.error("无身份证信息，不下发:{}", JSONUtil.toJsonStr(filing));
            return;
        }
        //备案场所
        Long venuesId = filing.getVenuesId();
        TbVenues tbVenues = tbVenuesService.getById(venuesId);
        if (tbVenues == null) {
            return;
        }
        //下发到等级比自己低或者同等级的场所设备
        Integer level = tbVenues.getLevel();
        List<TbTerminal> tbTerminalList = tbTerminalService.findByVenuesLevel(level);
        JSONObject params = builderParams(filing);
        tbTerminalList.forEach(tbTerminal -> {
            tbTerminalService.setCommand(CommandType.ISSUED_USER_COMMAND.getFuncId(), params, tbTerminal);
            tbTerminalService.startHeartCommand(tbTerminal.getSn());
        });
    }

    /**
     * 下发单个备案到单个设备
     *
     * @param filing
     * @param tbTerminal
     */
    public void issuedOneToTerminal(TbPersonFiling filing, TbTerminal tbTerminal, boolean startHeart) {
        if (filing.getJudgeState() != 1) {
            log.error("人员白名单未审核通过，不下发:{}", JSONUtil.toJsonStr(filing));
            return;
        }
        JSONObject params = builderParams(filing);
        tbTerminalService.setCommand(CommandType.ISSUED_USER_COMMAND.getFuncId(), params, tbTerminal);
        if (startHeart) {
            tbTerminalService.startHeartCommand(tbTerminal.getSn());
        }
    }

    /**
     * 排除设备下发
     *
     * @param id
     * @param sn
     */
    public void issuedNotTerminal(Long id, String sn) {
        TbPersonFiling filing = this.getById(id);
        if (filing.getJudgeState() != 1) {
            log.error("人员白名单未审核通过，不下发:{}", JSONUtil.toJsonStr(filing));
            return;
        }
        List<TbTerminal> tbTerminalList = tbTerminalService.findByVenuesId(filing.getVenuesId());
        log.info("排除设备下发人脸信息：{},{}", sn, JSONUtil.toJsonStr(filing));
        JSONObject params = builderParams(filing);
        tbTerminalList.stream().filter(t -> !StrUtil.equals(t.getSn(), sn)).forEach(tbTerminal -> {
            tbTerminalService.setCommand(CommandType.ISSUED_USER_COMMAND.getFuncId(), params, tbTerminal);
            tbTerminalService.startHeartCommand(tbTerminal.getSn());
        });
    }


    /**
     * 下发人员白名单到单个设备
     *
     * @param t
     */
    public void issued(TbTerminal t) {
        Long channelId = t.getChannelId();
        //通道
        TbVenues channel = tbVenuesService.getById(channelId);
        if (channel == null) {
            return;
        }
        //场所
        TbVenues venues = tbVenuesService.getById(channel.getPId());
        List<TbVenues> hightVenuesList = tbVenuesService.getHightLevel(venues.getLevel());
        List<TbVenues> venuesList = new ArrayList<>(hightVenuesList);
        venuesList.add(venues);
        List<TbPersonFiling> list = this.findByVenuesId(venuesList.stream().map(TbVenues::getId).collect(Collectors.toList()));
        //过滤审核通过的
        list = list.stream().filter(tbPersonFiling -> tbPersonFiling.getJudgeState() == 1).collect(Collectors.toList());
        list.forEach(tbPersonFiling -> {
            JSONObject params = builderParams(tbPersonFiling);
            tbTerminalService.setCommand(CommandType.ISSUED_USER_COMMAND.getFuncId(), params, t);
        });
    }

    /**
     * 根据场所查找备案人员
     *
     * @param venuesId
     * @return
     */
    public List<TbPersonFiling> findByVenuesId(Long venuesId) {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.lambda().eq(TbPersonFiling::getVenuesId, venuesId);
        return list(ew);
    }

    /**
     * 根据场所id上查找
     *
     * @param venuesIdList
     * @return
     */
    public List<TbPersonFiling> findByVenuesId(List<Long> venuesIdList) {
        if (venuesIdList.isEmpty()) {
            return Collections.emptyList();
        }
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.lambda().in(TbPersonFiling::getVenuesId, venuesIdList);
        return list(ew);
    }

    /**
     * 删除设备某个人的白名单
     */
    public void deleteTerminalWhilte(String pin, boolean setHeart) {
        TbPersonFiling filing = this.findByPin(pin);
        if (filing == null) {
            return;
        }
        //备案场所
        Long venuesId = filing.getVenuesId();
        TbVenues tbVenues = tbVenuesService.getById(venuesId);
        if (tbVenues == null) {
            return;
        }
        //删除级比自己低或者同等级的场所设备
        Integer level = tbVenues.getLevel();
        List<TbTerminal> tbTerminalList = tbTerminalService.findByVenuesLevel(level);
        JSONObject params = new JSONObject();
        params.putIfAbsent("pin", Collections.singleton(pin));
        tbTerminalList.forEach(tbTerminal -> {
            tbTerminalService.setCommand(CommandType.DELETE_USER_COMMAND.getFuncId(), params, tbTerminal);
            if (setHeart) {
                tbTerminalService.startHeartCommand(tbTerminal.getSn());
            }
        });

    }

    /**
     * 构建人员下发数据
     *
     * @param tbPersonFiling
     * @return
     */
    private JSONObject builderParams(TbPersonFiling tbPersonFiling) {
        WhitelistDTO dto = new WhitelistDTO();
        dto.setIdentity_number(tbPersonFiling.getIdCard())
                .setPin(tbPersonFiling.getPin())
                .setName(tbPersonFiling.getName());
        String photo = tbPersonFiling.getPhoto();
        if (StrUtil.isNotEmpty(photo)) {
            String filePath = new File(uploadConfig.rootFolder + tbPersonFiling.getPhoto().replace(SystemObject.config.getDomain(), "")).getAbsolutePath();
            dto.setPhoto(Base64Encoder.encode(FileUtil.readBytes(filePath)));
        }
        JSONObject params = new JSONObject();
        params.putIfAbsent("users", dto);
        return params;
    }


    /**
     * 查找部门的白名单
     *
     * @param deptId
     * @return
     */
    public List<TbPersonFiling> findByDeptId(Long deptId) {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.eq("dept_id", deptId);
        return list(ew);
    }

    public TbPersonFiling findByPin(String pin) {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.eq("pin", pin);
        return getOne(ew);
    }

    /**
     * 审核
     *
     * @param filing
     */
    public void judge(TbPersonFiling filing) {
        TbPersonFiling db = this.getById(filing.getId());
        if (db.getVenuesId() == null) {
            throw new AjaxError("请先选择备案场所");
        }
        int judgeState = filing.getJudgeState();
        db.setJudgeBy(StpUserUtil.getAdminName()).setJudgeContent(filing.getJudgeContent()).setEndTime(filing.getEndTime())
                .setJudgeTime(new Date()).setJudgeState(filing.getJudgeState());
        this.updateById(db);
        if (StrUtil.isNotEmpty(db.getPhoto())) {
            issued(db);
        }
        List<TbMiniUser> miniUsers = tbMiniUserService.findByPersonFilingId(filing.getId());
        miniUsers.parallelStream().forEach(tbMiniUser -> {
            MsgDataBO data = new MsgDataBO(
                    "您有一个备案审核提醒", judgeState == 2 ? "不通过" : "通过",
                    tbMiniUser.getName(),
                    db.getName() + "人员备案",
                    "审核时间：" + DateUtil.now() + "；" + filing.getJudgeContent());
            wxService.sendTemplateMsg(wxConfig.getJudgeResultTemplate(), tbMiniUser.getOpenid(), data);
        });
    }

    /**
     * 审核
     *
     * @param idCard
     * @param judgeStatus
     */
    public void judgeByIdCard(String idCard, int judgeStatus) {
        TbPersonFiling db = findByIdCard(idCard);
        if (db == null) {
            return;
        }
        db.setJudgeBy(StpUserUtil.getAdminName()).setJudgeTime(new Date()).setJudgeState(judgeStatus);
        this.updateById(db);
        if (StrUtil.isNotEmpty(db.getPhoto())) {
            issued(db);
        }
    }

    /**
     * 根据openid查找
     *
     * @param openid
     * @return
     */
    public TbPersonFiling findByOpenid(String openid) {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.eq("openid", openid);
        return getOne(ew);
    }

    public List<TbPersonFiling> getForWx(SoMap so) {
        return tbPersonFilingMapper.getForWx(so);
    }

    public List<TbPersonFiling> getOuterList(SoMap startPage) {
        return tbPersonFilingMapper.getOuterList(startPage);
    }

    /**
     * 批量导入
     *
     * @param file
     */
    public void importExcel(MultipartFile file) throws Exception {
        StaticLog.info("file size:{}", file.getSize());
        InputStream is = file.getInputStream();
        importPersonListener.list.clear();
        ExcelReader reader = EasyExcel.read(is, ImportPersonBO.class, importPersonListener).build();
        ReadSheet readSheet = EasyExcel.readSheet(0).build();
        reader.read(readSheet);
        reader.finish();
        is.close();
    }

    public void importFace(MultipartFile file) {
        if (file.getSize() == 0) {
            throw new AjaxError("上传文件不能为空");
        }
        String filename = file.getOriginalFilename();
        //创建临时目录
        String tempDir = uploadConfig.rootFolder + File.separator + RandomUtil.randomNumbers(6);
        String fileDir = tempDir + File.separator + StrUtil.trim(StrUtil.subBefore(filename, ".", true));
        File temFileDir = FileUtil.mkdir(tempDir);
        try (InputStream is = file.getInputStream()) {
            //解压
            ZipUtil.unzip(is, temFileDir, Charset.forName("gbk"));
        } catch (Exception e) {
            throw new AjaxError("压缩包解析错误");
        }
        //说明是压缩的是文件夹
        if (FileUtil.exist(fileDir)) {
            tempDir = fileDir;
        }
        List<String> files = FileUtil.listFileNames(tempDir);
        String fileFolder = new File(uploadConfig.rootFolder).getAbsolutePath() + "/" +
                uploadConfig.httpPrefix + "/face/";
        List<TbPersonFiling> personFilings = new ArrayList<>();
        for (String imageName : files) {
            String idCard = StrUtil.subBefore(imageName, ".", true);
            if (!IdcardUtil.isValidCard(idCard)) {
                throw new AjaxError(idCard + "身份证号码不正确");
            }
            TbPersonFiling tbPersonFiling = this.findByIdCard(idCard);
            if (tbPersonFiling == null) {
                throw new AjaxError(idCard + "人员不存在");
            }
            String newFilename = StrUtil.replace(imageName, idCard, RandomUtil.randomString(16));
            FileUtil.move(new File(tempDir + File.separator + imageName), new File(fileFolder + File.separator + newFilename), true);
            // 对外暴露的http路径
            String httpUrl = SystemObject.config.getDomain() + uploadConfig.httpPrefix + "/face/" + newFilename;
            tbPersonFiling.setPhoto(httpUrl);
            personFilings.add(tbPersonFiling);
        }
        this.updateBatchById(personFilings);
        FileUtil.del(temFileDir);
        //下发新的人脸到设备
        personFilings = personFilings.stream().filter(tbPersonFiling -> tbPersonFiling.getJudgeState() == 1).collect(Collectors.toList());
        taskService.addTask(new DelayIsuuedList(RandomUtil.randomNumbers(10), 2000, personFilings));
    }

    /**
     * 批量下发到设备端
     *
     * @param list
     */
    public void isuuedList(List<TbPersonFiling> list) {
        list.stream().collect(Collectors.groupingBy(TbPersonFiling::getVenuesId))
                .forEach((venuesId, venuesFillingList) -> {
                    TbVenues tbVenues = tbVenuesService.getById(venuesId);
                    issued(tbVenues, venuesFillingList);
                });
    }

    /**
     * 根据身份查找下发人员
     *
     * @param idCard
     */
    public void issued(String idCard) {
        TbPersonFiling tbPersonFiling = this.findByIdCard(idCard);
        if (tbPersonFiling == null) {
            return;
        }
        issued(tbPersonFiling);
    }

    /**
     * 下发到某场所及其等级低于等于场所的  设备
     *
     * @param tbVenues
     * @param personFilings
     */
    public void issued(TbVenues tbVenues, List<TbPersonFiling> personFilings) {
        List<TbTerminal> list = tbTerminalService.findByVenuesLevel(tbVenues.getLevel());
        list.stream().parallel().forEach(tbTerminal -> {
            personFilings.forEach(tbPersonFiling -> {
                JSONObject params = builderParams(tbPersonFiling);
                tbTerminalService.setCommand(CommandType.ISSUED_USER_COMMAND.getFuncId(), params, tbTerminal);
            });
            tbTerminalService.startHeartCommand(tbTerminal.getSn());
        });
    }

    /**
     * 获取系统中的商铺号
     *
     * @return
     */
    public List<String> getShopCodeList() {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.lambda().isNotNull(TbPersonFiling::getShopCode)
                .orderByAsc(TbPersonFiling::getCreateTime);
        List<TbPersonFiling> list = list(ew);
        return list.stream().map(TbPersonFiling::getShopCode).distinct().collect(Collectors.toList());
    }

    /**
     * 查找过期
     *
     * @return
     */
    public List<TbPersonFiling> findExpire() {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.le("date_format(end_time,'%Y-%m-%d')", DateUtil.today());
        return list(ew);
    }

    public String export(SoMap so) {
        List<TbPersonFiling> list = this.getList(so);
        String flieTypeFolder = "/export/";
        String currDateFolder = DateUtil.today();
        String fileName = "filing-" + RandomUtil.randomNumbers(6) + ".xlsx";
        String fileFolder = new File(uploadConfig.rootFolder).getAbsolutePath() +
                uploadConfig.httpPrefix + flieTypeFolder + currDateFolder + "/";
        if (!FileUtil.exist(fileFolder)) {
            FileUtil.mkdir(fileFolder);
        }
        EasyExcel.write(fileFolder + fileName, TbPersonFiling.class).sheet("人员备案信息")
                .doWrite(() -> list);
        return SystemObject.config.getDomain() + uploadConfig.httpPrefix + flieTypeFolder + currDateFolder + "/" + fileName;
    }

    public int getNeedJudgeNum(String departmentId) {
        QueryWrapper<TbPersonFiling> ew = new QueryWrapper<>();
        ew.lambda().eq(TbPersonFiling::getJudgeState, 0)
                .eq(StrUtil.isNotEmpty(departmentId), TbPersonFiling::getDepartmentId, departmentId);
        return this.count(ew);
    }

    public void rota(String id, int rota) {
        TbPersonFiling tbPersonFiling = this.getById(id);
        String photo = tbPersonFiling.getPhoto();
        String rootPath = uploadConfig.rootFolder;
        String domain = SystemObject.config.getDomain();
        String httpPrefix = StrUtil.sub(photo, 0, photo.lastIndexOf("/") + 1);
        String filePath = new File(rootPath + photo.replace(domain, "")).getAbsolutePath();
        String dirPath = StrUtil.sub(filePath, 0, filePath.lastIndexOf(File.separator) + 1);
        String newFilename = RandomUtil.randomNumbers(32) + ".jpg";
        ImgUtil.rotate(new File(filePath), rota, new File(dirPath + newFilename));
        FileUtil.del(new File(filePath));
        tbPersonFiling.setPhoto(httpPrefix + newFilename);
        this.updateById(tbPersonFiling);
        this.issued(tbPersonFiling);
    }
}
